package ca.recrown.islandsurvivalcraft.worldgeneration;

import org.bukkit.World;
import org.bukkit.block.Biome;
import org.bukkit.generator.ChunkGenerator.BiomeGrid;

import ca.recrown.islandsurvivalcraft.pathfinding.CoordinateTargetValidatable;
import ca.recrown.islandsurvivalcraft.pathfinding.DepthFirstSearch;

public class BiomePerIslandGenerator implements IslandBiomeGenerator, CoordinateTargetValidatable {
    private IslandMapGenerator islandMapGenerator;
    private BiomeSelector biomeSelector;
    private TemperatureMapGenerator temperatureMapGenerator;
    private int buildHeight;
    private World world;
    private DepthFirstSearch dfs;
    private Biome mainBiome;
    private Biome shoreBiome;
    private float temperature;
    
    public BiomePerIslandGenerator() {
        this.temperatureMapGenerator = new TemperatureMapGenerator();
        dfs = new DepthFirstSearch(islandMapGenerator);
    }
    
    @Override
    public void initialize(IslandMapGenerator mapGenerator, BiomeSelector biomeSelector) {
        this.islandMapGenerator = mapGenerator;
        this.biomeSelector = biomeSelector;
    }
    
    @Override
    public void setWorld(World world) {
        if (world.equals(this.world)) return;
        this.world = world;
        this.temperatureMapGenerator.setSeed(world.getSeed());
        this.buildHeight = world.getMaxHeight();
    }

    @Override
    public boolean isCoordinateTarget(int x, int y) {
        if (world.isChunkGenerated(x/16, y/16)) {
            Biome foundBiome = world.getBiome(x, 0, y);
            if (islandMapGenerator.isShore(x, y)) {
                shoreBiome = foundBiome;
            } else {
                mainBiome = foundBiome;
            }
            if (mainBiome != null && shoreBiome != null) {
                return true;
            }
        }
        return false;
    }

    @Override
    public void GenerateBiome(int chunkX, int chunkZ, int localX, int localZ, BiomeGrid biome) {
        int worldX = chunkX * 16 + localX;
        int worldZ = chunkZ * 16 + localZ;

        if (islandMapGenerator.isIsland(worldX, worldZ)) {
            if (mainBiome == null && shoreBiome == null) {
                if (!dfs.findTarget(this)) {
                    temperature = temperatureMapGenerator.getTemperature(worldX, worldZ);
                    mainBiome = biomeSelector.getLandBiome(temperature);
                    shoreBiome = biomeSelector.getShoreBiome(mainBiome, temperature);
                }
            }

        } else {
            mainBiome = null;
            shoreBiome = null;
        }
        for (int y = 0; y < buildHeight; y++) {
            Biome designatedBiome = mainBiome;
            if (mainBiome == null) {
                designatedBiome = Biome.OCEAN;
            } else if (islandMapGenerator.isShore(worldX, worldZ)) {
                    designatedBiome = shoreBiome;
            }

            biome.setBiome(localX, y, localZ, designatedBiome);
        }
    }
}