package ca.recrown.islandsurvivalcraft.worldgeneration;

import java.util.Random;

import org.bukkit.Material;
import org.bukkit.World;
import org.bukkit.generator.ChunkGenerator;

public class IslandSurvivalCraftWorldGenerator extends ChunkGenerator {
    private final BedrockGenerator bedrockGenerator;
    private final BiomeSelector biomeSelector;
    private final IslandBiomeGenerator biomeGenerator;
    private final IslandMapGenerator islandMapGenerator;

    public IslandSurvivalCraftWorldGenerator(IslandBiomeGenerator generator) {
        this.biomeSelector = new BiomeSelector();
        this.bedrockGenerator = new BedrockGenerator();
        this.islandMapGenerator = new IslandMapGenerator();
        this.biomeGenerator = generator;
    }
    
    public void initialize() {
        biomeSelector.initialize();
        biomeGenerator.initialize(islandMapGenerator, biomeSelector);
    }

    @Override
    public ChunkData generateChunkData(World world, Random random, int x, int z, BiomeGrid biome) {
        bedrockGenerator.setRandom(random);
        biomeSelector.setRandom(random);
        biomeGenerator.setWorld(world);
        islandMapGenerator.updateParameters(random, world.getSeaLevel(), 4);

        ChunkData chunk = createChunkData(world);
        for (int localX = 0; localX < 16; localX++) {
            for (int localZ = 0; localZ < 16; localZ++) {
                int worldX = 16*x + localX;
                int worldZ = 16*z + localZ;
                
                //generate bedrock layer
                int bedrockHeight = bedrockGenerator.getBedrockHeight(worldX, worldZ);
                chunk.setBlock(localX, bedrockHeight, localZ, Material.BEDROCK);

                //Set biomes
                biomeGenerator.GenerateBiome(x, z, localX, localZ, biome);
            }
        }
        return chunk;
    }
}